#!/bin/env python3

"""
TCX Speed Fixer

Usage: fix_tcx.py <filename> <new_filename> <adjustment_factor>

Recorded a bike ride with the wrong tire circumference set in your Speed sensor?
Given the circumference setting used to record the file, as well as the correct
circumference, this script will correct the TCX file.
"""

from decimal import Decimal
from numbers import Number
import sys
from xml.etree import ElementTree

XMLNS = {
    'tcx': 'http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2'}

ElementTree.register_namespace('', 'http://www.garmin.com/xmlschemas/TrainingCenterDatabase/v2')

def fix_tcx(filename: str,  new_filename: str, adjustment_factor: Number):
    def _fix_activity(activity):
        for lap in activity.findall('tcx:Lap', XMLNS):
            _fix_lap(lap)

    def _fix_lap(lap):
        for track in lap.findall('tcx:Track', XMLNS):
            _fix_track(track)

    def _fix_track(track):
        for trackpoint in track.findall('tcx:Trackpoint', XMLNS):
            _fix_trackpoint(trackpoint)

    def _fix_trackpoint(trackpoint):
        distance = trackpoint.find('tcx:DistanceMeters', XMLNS)
        distance.text = str(Decimal(distance.text) * adjustment_factor)

    tree = ElementTree.parse(filename)
    root = tree.getroot()
    activities = root.find('tcx:Activities', XMLNS)
    for activity in activities.findall('tcx:Activity', XMLNS):
        _fix_activity(activity)
    tree.write(new_filename)

if __name__ == '__main__':
    try:
        fix_tcx(sys.argv[1], sys.argv[2], Decimal(sys.argv[3]))
    except IndexError:
        print(__doc__)
